<?php

// Reminder: always indent with 4 spaces (no tabs).
// +---------------------------------------------------------------------------+
// | Navigation Manager - Navman Plugin for Geeklog                            |
// +---------------------------------------------------------------------------+
// | public_html/admin/plugins/navman/index.php                                |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 dengen - taharaxp AT gmail DOT com                     |
// |                                                                           |
// | Constructed with the Universal Plugin                                     |
// | Copyright (C) 2002 by the following authors:                              |
// | Tom Willett               -    twillett AT users DOT sourceforge DOT net  |
// | Blaine Lang               -    langmail AT sympatico DOT ca               |
// | The Universal Plugin is based on prior work by:                           |
// | Tony Bibbs                -    tony AT tonybibbs DOT com                  |
// | Modified by:                                                              |
// | mystral-kk                -    geeklog AT mystral-kk DOT net              |
// | dengen                    -    taharaxp AT gmail DOT com                  |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

require_once '../../../lib-common.php';
require_once '../../auth.inc.php';

if (!defined ('UC_SELECTED')) define('UC_SELECTED', (XHTML == '') ? 'selected' : 'selected="selected"');
if (!defined ('UC_CHECKED'))  define('UC_CHECKED',  (XHTML == '') ? 'checked'  : 'checked="checked"'  );
if (!defined ('UC_READONLY')) define('UC_READONLY', (XHTML == '') ? 'readonly' : 'readonly="readonly"');
if (!defined ('UC_DISABLED')) define('UC_DISABLED', (XHTML == '') ? 'disabled' : 'disabled="disabled"');

// Only let admin users access this page
if (!SEC_hasRights('navman.admin')) {
    // Someone is trying to illegally access this page
    COM_errorLog("Navman admin: Someone has tried to illegally access the Navman admin page. "
               . "User id: {$_USER['uid']}, Username: {$_USER['username']}, IP: $REMOTE_ADDR", 1);
    $display  = COM_siteHeader();
    $display .= COM_startBlock($LANG_NAVM['access_denied']);
    $display .= $LANG_NAVM['access_denied_msg'];
    $display .= COM_endBlock();
    $display .= COM_siteFooter(true);
    COM_output($display);
    exit;
}

$installed_version = DB_getItem($_TABLES['plugins'], 'pi_version', "pi_name = 'navman'");
$code_version = plugin_chkVersion_navman();
if (version_compare($installed_version, $code_version) < 0) {
    $display  = COM_siteHeader();
    $display .= COM_startBlock($LANG_NAVM['warning_updated']);
    $display .= $LANG_NAVM['instructions_update'];
    $display .= COM_endBlock();
    $display .= COM_siteFooter(true);
    COM_output($display);
    exit;
}

$MI = array(); // Array of values of one menuitem
$NM = array(); // Array of values of one navimenu

/**
* Set values of one menuitem in global array $MI
*/
function NAVM_setMI()
{
    global $_CONF, $MI;

    $url       = trim($_POST['url']);
    $icon_url  = trim($_POST['icon_url']);
    $menuorder = trim($_POST['menuorder']);
    $gid = COM_applyFilter($_POST['gid']);
    $gid = str_replace('ID', '', $gid);
    $MI = array(
        'mid'          => COM_applyFilter($_POST['mid']),
        'pmid'         => COM_applyFilter($_POST['pmid']),
        'gid'          => $gid,
        'is_enabled'   => ($_POST['is_enabled'] == 'on') ? 1 : 0,
        'type'         => $_POST['type'],
        'label'        => $_POST['title_fixation'],
        'label_var'    => $_POST['title_variable'],
        'php_function' => $_POST['php_function'],
        'url'          => empty($url)      ? '' : strip_tags($url),
        'icon_url'     => empty($icon_url) ? '' : strip_tags($icon_url),
        'tid'          => COM_applyFilter ($_POST['tid']),
        'menuorder'    => empty($menuorder) ? 0 : COM_applyFilter($menuorder, true),
        'pattern'      => $_POST['pattern'],
        'opt_match'    => COM_applyFilter($_POST['opt_match'], true),
        'class_name'   => COM_applyFilter($_POST['class_name']),
        'contentid'    => COM_applyFilter($_POST['contentid']),
        'owner_id'     => COM_applyFilter($_POST['owner_id'], true),
        'group_id'     => COM_applyFilter($_POST['group_id'], true),
        'perm_owner'   => $_POST['perm_owner'],
        'perm_group'   => $_POST['perm_group'],
        'perm_members' => $_POST['perm_members'],
        'perm_anon'    => $_POST['perm_anon'],
        'old_mid'      => COM_applyFilter($_POST['old_mid']),
    );

    // Convert array values to numeric permission values
    list($MI['perm_owner'], $MI['perm_group'], $MI['perm_members'], $MI['perm_anon'])
        = SEC_getPermissionValues($MI['perm_owner'], $MI['perm_group'], $MI['perm_members'], $MI['perm_anon']);
}

/**
* Validate global array $MI
*/
function NAVM_validateMI($mode)
{
    global $_TABLES, $LANG_NAVM_EDITOR, $LANG_NAVM, $MI;

    $retval = '';

    if (empty($MI['label'])) {
        $retval .= $LANG_NAVM_EDITOR['validate_message_3'] . '<br' . XHTML . '>';
    }

    if (empty($MI['mid'])) {
        $retval .= $LANG_NAVM_EDITOR['validate_message_4'] . '<br' . XHTML . '>';
    } else {
        $n = DB_count($_TABLES['navmmenuitems'], "mid", addslashes($MI['mid']));
        if (($n > 0) && ($mode != 'update')) {
            $retval .= $LANG_NAVM_EDITOR['validate_message_5'] . '<br' . XHTML . '>';
        }
    }

    if (!empty($MI['pattern']) && ($MI['opt_match'] == 2)) {
        if (@preg_match($MI['pattern'], 'test') === false) {
            $retval .= $LANG_NAVM_EDITOR['validate_message_6'] . '<br' . XHTML . '>';
        }
    }

    return $retval;
}


/**
* Check for topic access (need to handle 'all' and 'homeonly' as special cases)
*
* @param    string  $tid    ID for topic to check on
* @return   int             returns 3 for read/edit 2 for read only 0 for no access
*
*/
function NAVM_hasTopicAccess($tid)
{
    if (($tid == 'all') || ($tid == 'homeonly')) {
        $access = 3;
    } else {
        $access = SEC_hasTopicAccess($tid);
    }

    return $access;
}


/**
* Create menuitem id
*/
function NAVM_createMenuitemID($str = 'menuitem_')
{
    global $_TABLES;
    
    $i = 0;
    $n = 1;
    while ($n != 0) {
        $n = DB_count($_TABLES['navmmenuitems'], "mid", $str . (++$i));
    }
    $id = $str . $i;

    return $id;
}

/**
* Create navimenu id
*/
function NAVM_createNaviMenuID($str = 'navimenu_')
{
    global $_TABLES;
    
    $i = 0;
    $n = 1;
    while ($n != 0) {
        $n = DB_count($_TABLES['navmmenus'], "nid", $str . (++$i));
    }
    $id = $str . $i;

    return $id;
}

/**
* Shows the Menuitem editor
*
* This will show a Menuitem edit form.
*
* @param    string  $mid    ID of menuitem to edit
* @return   string          HTML for menuitem editor
*
*/
function NAVM_editMenuitem($mid, $mode='edit', $A=array())
{
    global $_CONF, $_GROUPS, $_TABLES, $_USER, $_PLUGINS, $_NAVM_CONF,
           $LANG_NAVM_EDITOR, $LANG_ACCESS, $LANG_ADMIN, $MESSAGE;

    $retval = '';

    if (($mode == 'edit') || ($mode == 'clone')) {
        if (empty($A)) {
            $result = DB_query("SELECT * FROM {$_TABLES['navmmenuitems']} WHERE mid ='" . addslashes($mid) . "'");
            $A = DB_fetchArray($result);
        } else {
            if (empty($A['mid'])) $A['mid'] = $A['old_mid'];
        }

        $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                                $A['perm_owner'],   $A['perm_group'],
                                $A['perm_members'], $A['perm_anon']);
        if (($access < 3) || (NAVM_hasTopicAccess($A['tid']) < 3)) {
            $retval .= COM_startBlock($LANG_ACCESS['accessdenied'], '', COM_getBlockTemplate('_msg_block', 'header'))
                     . $LANG_NAVM_EDITOR['message_access1']
                     . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
            COM_accessLog("User {$_USER['username']} tried to illegally create or edit menuitem $mid.");

            return $retval;
        }

        $selpmid = NAVM_makeSelBox($A['pmid']);
    }
    
    if ($mode == 'clone') {

        preg_match('/(.*)_clone_(.+)/', $A['mid'], $match);
        $label = empty($match[1]) ? $A['mid'] : $match[1];
        $A['mid'] = NAVM_createMenuitemID($label . '_clone_');
        $A['type'] = 'custom';
        $access = 3;
    }
    
    if ($mode == 'create') {

        $A['mid']          = NAVM_createMenuitemID();
        $A['pmid']         = '';
        $A['gid']          = 0;
        $A['is_enabled']   = 1;
        $A['type']         = 'custom';
        $A['label']        = '';
        $A['label_var']    = '';
        $A['php_function'] = '';
        $A['tid']          = 'all';
        $A['url']          = '';
        $A['icon_url']     = '';
        $A['menuorder']    = 0;
        $A['pattern']      = '';
        $A['opt_match']    = 0;
        $A['class_name']   = '';
        $A['contentid']    = '';

        $selpmid = NAVM_makeSelBox();

        $A['owner_id']     = (int) $_USER['uid'];
        if (isset($_GROUPS['Navman Admin'])) {
            $A['group_id'] = $_GROUPS['Navman Admin'];
        } else {
            $A['group_id'] = SEC_getFeatureGroup('navman.admin');
        }
        SEC_setDefaultPermissions($A, $_NAVM_CONF['default_permissions']);
        $access = 3;
    }

    $T = new Template($_NAVM_CONF['path_layout']);
    $T->set_file('editor', 'menuitemeditor.thtml');
    $T->set_var('xhtml', XHTML);
    $T->set_var('site_url',       $_CONF['site_url']);
    $T->set_var('site_admin_url', $_CONF['site_admin_url']);
    $T->set_var('icon_url', plugin_geticon_navman());
    $T->set_var('layout_url',     $_CONF['layout_url']);

    $retval .= COM_startBlock($LANG_NAVM_EDITOR['navmaneditor'], '',
                              COM_getBlockTemplate ('_admin_block', 'header'));

    $v = (($mode == 'create') || ($mode == 'clone') || 
          ($A['type'] == 'gldefault') || (($A['type'] == 'plugin') && in_array($A['mid'], $_PLUGINS))) ? UC_DISABLED : '';
    $T->set_var('delete_disabled', $v);

    foreach ($LANG_NAVM_EDITOR as $key => $val) {
        $T->set_var('lang_'.$key, $val);
    }

    $v = (($mode == 'create') or ($mode == 'clone')) ? 'save' : 'update';
    $T->set_var('lang_save', $LANG_NAVM_EDITOR[$v]);

    $T->set_var('val_title_fixation', stripslashes($A['label']));
    $T->set_var('val_title_variable', stripslashes($A['label_var']));
    $T->set_var('val_php_function',   stripslashes($A['php_function']));
    $T->set_var('val_is_enabled', ($A['is_enabled'] == 1) ? UC_CHECKED : '');
    $T->set_var('val_menuitemurl', $A['url']);
    $T->set_var('val_icon_url',    $A['icon_url']);
    $T->set_var('val_mid',         $A['mid']);
    $T->set_var('val_pmid',        $A['pmid']);
    $T->set_var('selpmid',         $selpmid);
    $T->set_var('val_gid',         $A['gid']);
    $T->set_var('val_contentid',   $A['contentid']);
    $T->set_var('val_old_mid',     $A['mid']);
    $T->set_var('val_menuorder',   $A['menuorder']);
    $T->set_var('val_type',        $A['type']);
    $T->set_var('val_pattern',     stripslashes($A['pattern']));
    $T->set_var('val_opt_match_0', ($A['opt_match'] == 0)  ? UC_SELECTED : '');
    $T->set_var('val_opt_match_1', ($A['opt_match'] == 1)  ? UC_SELECTED : '');
    $T->set_var('val_opt_match_2', ($A['opt_match'] == 2)  ? UC_SELECTED : '');
    $T->set_var('mitem_group_list',  NAVM_GetMenuitemGroupList($A['gid']));
    $T->set_var('val_class_name',  stripslashes($A['class_name']));
    $v = (($A['type'] == 'gldefault') || ($A['type'] == 'plugin')) ? UC_READONLY : '';
    $T->set_var('mid_readonly', $v);
    $T->set_var($A['tid'].'_selected', UC_SELECTED);
    $T->set_var('topic_options', COM_topicList('tid,topic', $A['tid'], 1, true));
    $T->set_var('lang_type', $LANG_NAVM_EDITOR[ 'type_' . $A['type'] ]);
    $T->set_var($A['type'].'_selected', UC_SELECTED);

    // user access info
    $T->set_var('lang_accessrights', $LANG_ACCESS['accessrights']);
    $T->set_var('lang_owner', $LANG_ACCESS['owner']);
    $ownername = COM_getDisplayName($A['owner_id']);
    $T->set_var('owner_username', DB_getItem($_TABLES['users'], 'username', "uid = {$A['owner_id']}"));
    $T->set_var('owner_name', $ownername);
    $T->set_var('owner', $ownername);
    $T->set_var('owner_id', $A['owner_id']);
    $T->set_var('lang_group', $LANG_ACCESS['group']);
    $T->set_var('group_dropdown', SEC_getGroupDropdown($A['group_id'], $access));
    $T->set_var('lang_permissions', $LANG_ACCESS['permissions']);
    $T->set_var('lang_permissionskey', $LANG_ACCESS['permissionskey']);
    $T->set_var('permissions_editor', 
        SEC_getPermissionsHTML($A['perm_owner'], $A['perm_group'], $A['perm_members'], $A['perm_anon']));
    $T->set_var('lang_lockmsg', $LANG_ACCESS['permmsg']);
    if (!defined(CSRF_TOKEN)) define('CSRF_TOKEN', 'token');
    $T->set_var('gltoken_name', CSRF_TOKEN);
    $T->set_var('gltoken', SEC_createToken());

    $T->parse('output', 'editor');
    $retval .= $T->finish($T->get_var('output'));

    $retval .= COM_endBlock(COM_getBlockTemplate('_admin_block', 'footer'));

    return $retval;
}

/**
* Display the list of the menuitems
*
* @return   string                  HTML error message
*
*/
function NAVM_listMenuitems($current='all')
{
    global $_CONF, $_TABLES, $LANG_ADMIN, $LANG_NAVM_EDITOR, $LANG_NAVM, $_IMAGE_TYPE;

    require_once $_CONF['path_system'] . 'lib-admin.php';

    $retval = '';
    
    NAVM_reorderMenuitems();

    $header_arr   = array(
                    array('text' => $LANG_ADMIN['edit'],       'field' => 'edit',      'sort' => false),
                    array('text' => $LANG_NAVM['order'],       'field' => 'menuorder', 'sort' => true),
                    array('text' => $LANG_ADMIN['title'],      'field' => 'label',     'sort' => true),
                    array('text' => $LANG_NAVM['id'],          'field' => 'mid',       'sort' => true),
                    array('text' => $LANG_NAVM['classname'],   'field' => 'class_name','sort' => true),
                    array('text' => $LANG_ADMIN['type'],       'field' => 'type',      'sort' => true),
                    array('text' => $LANG_ADMIN['topic'],      'field' => 'tid',       'sort' => true),
    );

    $defsort_arr = array('field' => 'menuorder', 'direction' => 'asc');

    $menu_arr = array(
                    array('url'  => $_CONF['site_admin_url'] . '/plugins/navman/index.php',
                          'text' => $LANG_NAVM['list_navimenus']),
                    array('url'  => $_CONF['site_admin_url'] . '/plugins/navman/index.php?mode=create',
                          'text' => $LANG_NAVM['create_menuitem']),
                    array('url'  => $_CONF['site_admin_url'],
                          'text' => $LANG_ADMIN['admin_home'])
    );

    $filter = $LANG_NAVM['select_mitemgroup'] . ':&nbsp;' . NAVM_GetMenuitemGroupList($current, true, true) . '&nbsp;&nbsp;';


    $sql = "SELECT * FROM {$_TABLES['navmmenuitems']} WHERE menuorder > 0";
    $sql .= ($current != 'all') ? " AND gid = '$current'" : '';

    $retval .= COM_startBlock($LANG_NAVM['manager'], '', COM_getBlockTemplate('_admin_block', 'header'));

    $retval .= ADMIN_createMenu($menu_arr, $LANG_NAVM['instructions'], plugin_geticon_navman());

    $text_arr = array('has_extras' => true,
                      'form_url'   => $_CONF['site_admin_url'] . "/plugins/navman/index.php?mode=listmenuitems");

    $query_arr = array('table'          => 'menuitems',
                       'sql'            => $sql,
                       'query_fields'   => array('label'),
                       'default_filter' => COM_getPermSql('AND'));

    $extra = array('current' => 'ID' . $current);

    $retval .= ADMIN_list("menuitems", "NAVM_getListField_Menuitems", $header_arr, $text_arr,
                           $query_arr, $defsort_arr, $filter, $extra);

    $retval .= COM_endBlock(COM_getBlockTemplate('_admin_block', 'footer'));

    return $retval;
}


/**
* Saves a menuitem
*
* @return   string                  HTML error message
*
*/
function NAVM_saveMenuitems($mode)
{
    global $_TABLES, $LANG_NAVM_EDITOR, $MI;

    $retval = '';

    if ((!empty($MI['mid'])) && (DB_count($_TABLES['navmmenuitems'], 'mid', addslashes($MI['mid'])) > 0)) {
        $result = DB_query("SELECT owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon "
                         . "FROM {$_TABLES['navmmenuitems']} WHERE mid = '" . addslashes($MI['mid']) . "'");
        $A = DB_fetchArray($result);
        $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                                $A['perm_owner'],   $A['perm_group'], 
                                $A['perm_members'], $A['perm_anon']);
    } else {
        $access = SEC_hasAccess($MI['owner_id'],     $MI['group_id'],
                                $MI['perm_owner'],   $MI['perm_group'],
                                $MI['perm_members'], $MI['perm_anon']);
    }

    if (($access < 3) || (!NAVM_hasTopicAccess($MI['tid']))) {
        $retval .= COM_startBlock($LANG_NAVM_EDITOR['access_denied'], '', COM_getBlockTemplate('_msg_block', 'header'))
                 . $LANG_NAVM_EDITOR['message_access2']
                 . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        COM_accessLog("User {$_USER['username']} tried to illegally submit or edit menuitem {$MI['mid']}.");
        return $retval;
    }

    $MI['mid']          = addslashes(COM_stripslashes(strip_tags($MI['mid'])));
    $MI['pmid']         = addslashes(COM_stripslashes(strip_tags($MI['pmid'])));
    $MI['label']        = addslashes(COM_stripslashes(strip_tags($MI['label'])));
    $MI['label_var']    = addslashes(COM_stripslashes(strip_tags($MI['label_var'])));
    $MI['php_function'] = addslashes(COM_stripslashes(strip_tags($MI['php_function'])));
    $MI['class_name']   = addslashes(COM_stripslashes(strip_tags($MI['class_name'])));
    $MI['contentid']    = addslashes(COM_stripslashes(strip_tags($MI['contentid'])));
    $MI['url']          = addslashes($MI['url']);
    $MI['icon_url']     = addslashes($MI['icon_url']);
    $MI['pattern']      = addslashes(str_replace('\\', '\\\\', $MI['pattern']));
    $MI['type']         = addslashes($MI['type']);
    $MI['tid']          = addslashes($MI['tid']);
    $MI['is_enabled']   = (int) $MI['icon_url'];
    $MI['gid']          = (int) $MI['gid'];
    $MI['menuorder']    = (int) $MI['menuorder'];
    $MI['owner_id']     = (int) $MI['owner_id'];
    $MI['group_id']     = (int) $MI['group_id'];
    $MI['perm_owner']   = (int) $MI['perm_owner'];
    $MI['perm_group']   = (int) $MI['perm_group'];
    $MI['perm_members'] = (int) $MI['perm_members'];
    $MI['perm_anon']    = (int) $MI['perm_anon'];
    $MI['opt_match']    = (int) $MI['opt_match'];

    if ($mode == 'update') {
        if ($MI['old_mid'] != $MI['mid']) {
            DB_delete($_TABLES['navmmenuitems'], 'mid', $MI['old_mid']);
            $sql = "INSERT INTO {$_TABLES['navmmenuitems']} "
             .  '(mid, is_enabled, type, label, label_var, gid, php_function, url, '
             .  'icon_url, tid, menuorder, '
             .  'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
             .  'pattern, pmid, class_name, opt_match, contentid) '
             .  "VALUES ('" . $MI['mid']          . "',"  . $MI['is_enabled'] . ",'"  . $MI['type']       . "','" 
                            . $MI['label']        . "','"   . $MI['label_var']. "',"  . $MI['gid']        . ",'" 
                            . $MI['php_function'] . "','" . $MI['url']        . "','" . $MI['icon_url']   . "','" 
                            . $MI['tid']          . "',"  . $MI['menuorder']  . ","   . $MI['owner_id']   . "," 
                            . $MI['group_id']     . ","   . $MI['perm_owner'] . ","   . $MI['perm_group'] . "," 
                            . $MI['perm_members'] . ","   . $MI['perm_anon']  . ",'"  . $MI['pattern']    . "','"
                            . $MI['pmid']         . "','" . $MI['class_name'] . "',"  . $MI['opt_match']  . ",'"
                            . $MI['contentid']    . "')";
            DB_query($sql);

            // With a change of mid, I change pmid.
            $result = DB_query("SELECT mid FROM {$_TABLES['navmmenuitems']} WHERE pmid ='" . addslashes($MI['old_mid']) . "'");
            while ($B = DB_fetchArray($result)) {
                DB_query("UPDATE {$_TABLES['navmmenuitems']} SET pmid = '" . $MI['mid'] . "' WHERE mid = '" . addslashes($B['mid']) . "'");
            }

        } else {
            DB_save($_TABLES['navmmenuitems'],
                'mid, is_enabled, type, label, label_var, gid, php_function, url, '
              . 'icon_url, tid, menuorder, '
              . 'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
              . 'pattern, pmid, class_name, opt_match, contentid',
                "'" . $MI['mid']          . "',"  . $MI['is_enabled'] . ",'"  . $MI['type']       . "','" 
                    . $MI['label']        . "','" . $MI['label_var']  . "',"  . $MI['gid']        . ",'" 
                    . $MI['php_function'] . "','" . $MI['url']        . "','" . $MI['icon_url']   . "','" 
                    . $MI['tid']          . "',"  . $MI['menuorder']  . ","   . $MI['owner_id']   . "," 
                    . $MI['group_id']     . ","   . $MI['perm_owner'] . ","   . $MI['perm_group'] . "," 
                    . $MI['perm_members'] . ","   . $MI['perm_anon']  . ",'"  . $MI['pattern']    . "','" 
                    . $MI['pmid']         . "','" . $MI['class_name'] . "',"  . $MI['opt_match']  . ",'" 
                    . $MI['contentid']    . "'");
        }

    } else {
        $sql = "INSERT INTO {$_TABLES['navmmenuitems']} "
         .  '(mid, is_enabled, type, label, label_var, gid, php_function, url, '
         .  'icon_url, tid, menuorder, '
         .  'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
         .  'pattern, pmid, class_name, opt_match, contentid) '
         .  "VALUES ('" . $MI['mid']          . "',"  . $MI['is_enabled'] . ",'"  . $MI['type']       . "','" 
                        . $MI['label']        . "','" . $MI['label_var']  . "',"  . $MI['gid']        . ",'" 
                        . $MI['php_function'] . "','" . $MI['url']        . "','" . $MI['icon_url']   . "','" 
                        . $MI['tid']          . "',"  . $MI['menuorder']  . ","   . $MI['owner_id']   . "," 
                        . $MI['group_id']     . ","   . $MI['perm_owner'] . ","   . $MI['perm_group'] . "," 
                        . $MI['perm_members'] . ","   . $MI['perm_anon']  . ",'"  . $MI['pattern']    . "','"
                        . $MI['pmid']         . "','" . $MI['class_name'] . "',"  . $MI['opt_match']  . ",'"
                        . $MI['contentid']    . "')";
        DB_query($sql);
    }

    return $retval;
}


/**
* 
*/
function NAVM_makeSelBox($preset_id = "")
{
    global $_TABLES;

    $retval = '<select name="pmid">' . LB;
    $retval .= '<option value="">----</option>' . LB;
    $sql = "SELECT mid FROM {$_TABLES['navmmenuitems']} WHERE pmid = '' ORDER BY menuorder";
    $result = DB_query($sql);
    while (list($mid) = DB_fetchArray($result)) {
        $sel = ($mid == $preset_id) ? ' ' . UC_SELECTED : '';
        $retval .= "<option value=\"$mid\"$sel>$mid</option>" . LB;
        $A = NAVM_getChildTreeArray($mid);
        foreach ($A as $B) {
            $level = NAVM_getMenuLevel($B['mid']);
            $prefix = '';
            for ($i = 0 ; $i < $level; $i++) {
                $prefix .= '&nbsp;&nbsp;&nbsp;';
            }
            $catpath = $prefix . $B['mid'];
            $sel = ($B['mid'] == $preset_id) ? ' ' . UC_SELECTED : '';
            $retval .= "<option value=\"" . $B['mid'] . "\"$sel>$catpath</option>" . LB;
        }
    }
    $retval .= '</select>' . LB;
    return $retval;
}


/**
* 
*/
function NAVM_getChildTreeArray($sel_id="", $parray = array(), $orderby = "ASC")
{
    global $_TABLES;

    $sql = "SELECT * FROM {$_TABLES['navmmenuitems']} "
         . "WHERE pmid = '" . addslashes($sel_id) . "' ORDER BY menuorder $orderby";
    $result = DB_query($sql);
    if (DB_numRows($result) == 0) return $parray;
    while ($A = DB_fetchArray($result)) {
        array_push($parray, $A);
        $parray = NAVM_getChildTreeArray($A['mid'], $parray, $orderby);
    }
    return $parray;
}

/**
* Select navigation and create array
*/
function NAVM_getChildTreeArray_2($gid, $sel_id='', $parray=array(), $orderby='ASC')
{
    global $_TABLES;

    $sql = "SELECT * FROM {$_TABLES['navmmenuitems']} "
         . "WHERE pmid = '" . addslashes($sel_id) . "' "
         . "AND gid = '" . addslashes($gid) . "' "
         . "ORDER BY menuorder $orderby";
    $result = DB_query($sql);
    if (DB_numRows($result) == 0) return $parray;
    while ($A = DB_fetchArray($result)) {
        array_push($parray, $A);
        $parray = NAVM_getChildTreeArray_2($gid, $A['mid'], $parray, $orderby);
    }
    return $parray;
}


/**
* Re-orders all menuitems in steps of 10
*/
function NAVM_reorderMenuitems()
{
    global $_TABLES;

    $result = DB_query("SELECT DISTINCT gid FROM {$_TABLES['navmmenuitems']} ORDER BY gid ASC");
    while ($A = DB_fetchArray($result)) {
        $B = NAVM_getChildTreeArray_2($A['gid']);
        foreach ($B as $C) {
            $order += 10;
            if ($C['menuorder'] != $order) {
                DB_query("UPDATE {$_TABLES['navmmenuitems']} SET menuorder = '$order' WHERE mid = '" . addslashes($C['mid']) . "'");
            }
        }
    }
}

/**
* Re-orders all navimenus in steps of 10
*/
function NAVM_reorderNaviMenus()
{
    global $_TABLES;

    $result = DB_query("SELECT nid FROM {$_TABLES['navmmenus']} ORDER BY navorder ASC");
    while ($A = DB_fetchArray($result)) {
        $order += 10;
        if ($A['navorder'] != $order) {
            DB_query("UPDATE {$_TABLES['navmmenus']} SET navorder = '$order' WHERE nid = '" . addslashes($A['nid']) . "'");
        }
    }
}

/**
* Move menuitem UP and Down
*/
function NAVM_moveMenuitem($gid='')
{
    global $_TABLES;

    $mid   = addslashes(COM_applyFilter($_GET['mid']));
    $where = COM_applyFilter($_GET['where']);
    $menuorder = DB_getItem($_TABLES['navmmenuitems'], 'menuorder', "mid = '$mid'");
    $pmid      = DB_getItem($_TABLES['navmmenuitems'], 'pmid',      "mid = '$mid'");

    if ($where == 'up') {
        $A = NAVM_getChildTreeArray_2($gid, '', array(), 'DESC');
        foreach ($A as $B) {
            $order = $B['menuorder'] - 1;
            if (($B['menuorder'] < $menuorder) && ($B['pmid'] == $pmid)) break;
        }
    } else {
        $A = NAVM_getChildTreeArray_2($gid);
        foreach ($A as $B) {
            $order = $B['menuorder'] + 1;
            if (($B['menuorder'] > $menuorder) && ($B['pmid'] == $pmid)) break;
        }
    }

    DB_query("UPDATE {$_TABLES['navmmenuitems']} SET menuorder = $order WHERE mid = '$mid'");
}


/**
* Move navimenu UP and Down
*/
function NAVM_moveNaviMenu($nid='')
{
    global $_TABLES;

    $nid = addslashes($nid);
    $where = COM_applyFilter($_GET['where']);
    $navorder = DB_getItem($_TABLES['navmmenus'], 'navorder', "nid = '$nid'");

    if (!empty($navorder)) {
        if ($where == 'up') {
            if ($navorder>=11) $navorder -= 11;
        } else {
            $navorder += 11;
        }
        DB_query("UPDATE {$_TABLES['navmmenus']} SET navorder = $navorder WHERE nid = '$nid'");
    }
}


/**
* Enable and Disable menuitem
*/
function NAVM_changeMenuitemStatus()
{
    global $_TABLES;
    
    $result = DB_query("SELECT mid FROM {$_TABLES['navmmenuitems']}");
    while ($A = DB_fetchArray($result)) {
        $name = 'mid_' . $A['mid'];
        $update = 'update_mid_' . $A['mid'];
        $v = COM_applyFilter($_POST[$update], true);
        if ($v == 1) {
            $v = COM_applyFilter($_POST[$name], true);
            DB_query("UPDATE {$_TABLES['navmmenuitems']} SET is_enabled = $v WHERE mid = '" . addslashes($A['mid']) . "'");
            break;
        }
    }
}


/**
* Enable and Disable navimenu
*/
function NAVM_changeNaviMenuStatus()
{
    global $_TABLES;
    
    $result = DB_query("SELECT nid FROM {$_TABLES['navmmenus']}");
    while ($A = DB_fetchArray($result)) {
        $name = 'nid_' . $A['nid'];
        $update = 'update_' . $A['nid'];
        $v = COM_applyFilter($_POST[$update], true);
        if ($v == 1) {
            $v = COM_applyFilter($_POST[$name], true);
            DB_query("UPDATE {$_TABLES['navmmenus']} SET is_enabled = $v WHERE nid = '" . addslashes($A['nid']) . "'");
            break;
        }
    }
}

/**
* Delete a menuitem
*
* @param    string  $mid    id of menuitem to delete
* @return   string          HTML error message
*
*/
function NAVM_deleteMenuitem($mid)
{
    global $_TABLES, $_USER;

    $retval = '';

    $result = DB_query("SELECT tid,owner_id,group_id,perm_owner,perm_group,perm_members,perm_anon "
                     . "FROM {$_TABLES['navmmenuitems']} WHERE mid ='" . addslashes($mid) . "'");
    $A = DB_fetchArray($result);
    $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                            $A['perm_owner'],   $A['perm_group'],
                            $A['perm_members'], $A['perm_anon']);
    if (($access < 3) || (NAVM_hasTopicAccess ($A['tid']) < 3)) {

        $retval .= COM_startBlock($LANG_NAVM_EDITOR['access_denied'], '', COM_getBlockTemplate ('_msg_block', 'header'))
                 . $LANG_NAVM_EDITOR['message_access3']
                 . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        COM_accessLog("User {$_USER['username']} tried to illegally delete menuitem $mid.");

        return $retval;
    }

    DB_delete($_TABLES['navmmenuitems'], 'mid', $mid);
    if (!DB_error()) {
        $result = DB_query("SELECT mid FROM {$_TABLES['navmmenuitems']} WHERE pmid ='" . addslashes($mid) . "'");
        while ($B = DB_fetchArray($result)) {
            DB_query("UPDATE {$_TABLES['navmmenuitems']} SET pmid = '' WHERE mid = '" . addslashes($B['mid']) . "'");
        }
    }

    return $retval;
}

/**
* Delete a navimenu
*
* @param    string  $mid    id of menuitem to delete
* @return   string          HTML error message
*
*/
function NAVM_deleteNaviMenu($nid)
{
    global $_TABLES, $_USER;

    $retval = '';

    $result = DB_query("SELECT owner_id,group_id,perm_owner,perm_group,perm_members,perm_anon "
                     . "FROM {$_TABLES['navmmenus']} WHERE nid ='" . addslashes($nid) . "'");
    $A = DB_fetchArray($result);
    $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                            $A['perm_owner'],   $A['perm_group'],
                            $A['perm_members'], $A['perm_anon']);
    if ($access < 3) {

        $retval .= COM_startBlock($LANG_NAVM_EDITOR['access_denied'], '', COM_getBlockTemplate ('_msg_block', 'header'))
                 . $LANG_NAVM_EDITOR['message_access3']
                 . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        COM_accessLog("User {$_USER['username']} tried to illegally delete navimenu $nid.");

        return $retval;
    }

    DB_delete($_TABLES['navmmenus'], 'nid', $nid);

    return $retval;
}


/**
* 
*/
function NAVM_getMenuLevel($mid)
{
    global $_TABLES;

    $pmid = DB_getItem($_TABLES['navmmenuitems'], 'pmid', "mid = '" . addslashes($mid) . "'");
    if ($pmid != '') {
        return 1 + NAVM_getMenuLevel($pmid);
    }
    return 0;
}

/**
* Get ListFields of Menuitems
*/
function NAVM_getListField_Menuitems($fieldname, $fieldvalue, $A, $icon_arr, $extra=array())
{
    global $_CONF, $LANG_ADMIN, $_PLUGINS, $_NAVM_CONF, $LANG_NAVM_EDITOR, $MESSAGE, $NAVM_CSRF_TOKEN;
    
    $retval = false;

    $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                            $A['perm_owner'],   $A['perm_group'],
                            $A['perm_members'], $A['perm_anon']);

    if (($access > 0) && (NAVM_hasTopicAccess ($A['tid']) > 0)) {

        $token  = "";
        if (version_compare(VERSION, '1.5') >= 0) {
            $token  = "&amp;" . CSRF_TOKEN . "=" . $NAVM_CSRF_TOKEN;
        }
        $param_gid = (!empty($extra['current'])) ? '&amp;gid='.$extra['current'] : '';

        switch($fieldname) {
            case 'edit':
                if ($access == 3) {
                    $retval = "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=edit&amp;mid={$A['mid']}\">{$icon_arr['edit']}</a>" . LB
                            . "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=clone&amp;mid={$A['mid']}\">{$icon_arr['copy']}</a>" . LB;
                }

                if ( ($access == 3 && ($A['type'] == 'custom')) || 
                     ($access == 3 && ($A['type'] == 'plugin') && !in_array($A['mid'], $_PLUGINS)) ) {
                    $icon   = "<img src=\"{$_CONF['site_url']}/navman/images/delete.png\" "
                            . "alt=\"{$LANG_ADMIN['delete']}\" title=\"{$LANG_ADMIN['delete']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=delete&amp;mid={$A['mid']}" . $param_gid
                            . $token . "\" onclick=\"return confirm('{$MESSAGE[76]}');\">$icon</a>";
                }

                break;

            case 'menuorder':
                if ($access == 3) {
                    $iconup  = "<img src=\"{$_CONF['site_url']}/navman/images/arrow-up.png\" "
                             . "alt=\"{$LANG_NAVM_EDITOR['move_up']}\" title=\"{$LANG_NAVM_EDITOR['move_up']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                             . "?mode=listmenuitems" . $param_gid . "&amp;mid={$A['mid']}&amp;where=up" . $token . "\">$iconup</a>";

                    $icondn  = "<img src=\"{$_CONF['site_url']}/navman/images/arrow-dn.png\" "
                             . "alt=\"{$LANG_NAVM_EDITOR['move_down']}\" title=\"{$LANG_NAVM_EDITOR['move_down']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                             . "?mode=listmenuitems" . $param_gid . "&amp;mid={$A['mid']}&amp;where=dn" . $token . "\">$icondn</a>";

                    $retval .= "&nbsp;";
                    $retval .= $A['menuorder'];
                }
                break;

            case 'label':
                $retval = (!empty($A['label'])) ? stripslashes($A['label']) : '(' . $A['mid'] . ')';
                if ($access == 3) {
                    $switch = ($A['is_enabled'] == 1) ? UC_CHECKED : '';
                    $val = ($A['is_enabled'] == 1) ? 1 : 0;
                    $name = 'mid_' . $A['mid'];
                    $update = 'update_mid_' . $A['mid'];
                    $retval = "<input type=\"checkbox\" name=\"$name\" onclick=\"this.form.elements['$update'].value=1; submit()\" value=\"1\" $switch>" . $retval;
                    $retval .= "<input type=\"hidden\" name=\"$update\" value=\"0\"" . XHTML . ">";
                    $retval .= "<input type=\"hidden\" name=\"gid\" value=\"" . $extra['current'] . "\"" . XHTML . ">";
                    $retval .= "<input type=\"hidden\" name=\"" . CSRF_TOKEN . "\" value=\"$NAVM_CSRF_TOKEN\"" . XHTML . ">";
                }
                break;

            case 'mid':
                $level = NAVM_getMenuLevel($A['mid']);
                $retval = '';
                for ($i = 0 ; $i < $level; $i++) {
                    $retval .= '&nbsp;&nbsp;&nbsp;';
                }
                $retval .= $A['mid'];
                break;

            default:
                $retval = $fieldvalue;
                break;
        }
    }
    return $retval;
}


/**
* Set values of one navimenu in global array $NM
*/
function NAVM_setNM()
{
    global $_CONF, $NM;

    $navorder = trim($_POST['navorder']);
    $gid = COM_applyFilter($_POST['gid']);
    $gid = str_replace('ID', '', $gid);
    $NM = array(
        'nid'          => COM_applyFilter($_POST['nid']),
        'gid'          => $gid,
        'is_enabled'   => ($_POST['is_enabled'] == 'on') ? 1 : 0,
        'name'         => $_POST['navimenuname'],
        'php_function' => $_POST['php_function'],
        'class_name'   => COM_applyFilter($_POST['class_name']),
        'template_name' => COM_applyFilter($_POST['template_name']),
        'var_name'     => COM_applyFilter($_POST['var_name'], true),
        'navorder'     => empty($navorder) ? 0 : COM_applyFilter($navorder, true),
        'pattern'      => $_POST['pattern'],
        'render'       => COM_applyFilter($_POST['render'], true),
        'render_for'   => COM_applyFilter($_POST['render_for'], true),
        'opt_match'    => COM_applyFilter($_POST['opt_match'], true),
        'owner_id'     => COM_applyFilter($_POST['owner_id'], true),
        'group_id'     => COM_applyFilter($_POST['group_id'], true),
        'perm_owner'   => $_POST['perm_owner'],
        'perm_group'   => $_POST['perm_group'],
        'perm_members' => $_POST['perm_members'],
        'perm_anon'    => $_POST['perm_anon'],
        'old_nid'      => COM_applyFilter($_POST['old_nid']),
    );

    // Convert array values to numeric permission values
    list($NM['perm_owner'], $NM['perm_group'], $NM['perm_members'], $NM['perm_anon'])
        = SEC_getPermissionValues($NM['perm_owner'], $NM['perm_group'], $NM['perm_members'], $NM['perm_anon']);
}

/**
* Validate global array $NM
*/
function NAVM_validateNM($mode)
{
    global $_TABLES, $LANG_NAVM_EDITOR, $LANG_NAVM, $NM;

    $retval = '';

    if (empty($NM['name'])) {
        $retval .= $LANG_NAVM_EDITOR['validate_message_7'] . '<br' . XHTML . '>';
    }

    if (empty($NM['nid'])) {
        $retval .= $LANG_NAVM_EDITOR['validate_message_8'] . '<br' . XHTML . '>';
    } else {
        $n = DB_count($_TABLES['navmmenus'], "nid", addslashes($NM['nid']));
        if (($n > 0) && ($mode != 'updatemenu')) {
            $retval .= $LANG_NAVM_EDITOR['validate_message_9'] . '<br' . XHTML . '>';
        }
    }

    if (!empty($NM['pattern']) && ($NM['opt_match'] == 2)) {
        if (@preg_match($NM['pattern'], 'test') === false) {
            $retval .= $LANG_NAVM_EDITOR['validate_message_6'] . '<br' . XHTML . '>';
        }
    }

    return $retval;
}

/**
* Shows the NaviMenu editor
*
* This will show a NaviMenu edit form.
*
*/
function NAVM_editNaviMenu($nid, $mode='editmenu', $A=array())
{
    global $_CONF, $_GROUPS, $_TABLES, $_USER, $_PLUGINS, $_NAVM_CONF,
           $LANG_NAVM_EDITOR, $LANG_ACCESS, $LANG_ADMIN, $MESSAGE;

    $retval = '';

    if (($mode == 'editmenu') || ($mode == 'clonemenu')) {
        if (empty($A)) {
            $result = DB_query("SELECT * FROM {$_TABLES['navmmenus']} WHERE nid ='" . addslashes($nid) . "'");
            $A = DB_fetchArray($result);
        } else {
            if (empty($A['nid'])) $A['nid'] = $A['old_nid'];
        }

        $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                                $A['perm_owner'],   $A['perm_group'],
                                $A['perm_members'], $A['perm_anon']);
        if ($access < 3) {
            $retval .= COM_startBlock($LANG_ACCESS['accessdenied'], '', COM_getBlockTemplate('_msg_block', 'header'))
                     . $LANG_NAVM_EDITOR['message_access1']
                     . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
            COM_accessLog("User {$_USER['username']} tried to illegally create or edit navimenu $nid.");

            return $retval;
        }
    }
    
    if ($mode == 'clonemenu') {

        preg_match('/(.*)_clone_(.+)/', $A['nid'], $match);
        $label = empty($match[1]) ? $A['nid'] : $match[1];
        $A['nid'] = NAVM_createNaviMenuID($label . '_clone_');
        $access = 3;
    }
    
    if ($mode == 'createmenu') {

        $A['nid']           = NAVM_createNaviMenuID();
        $A['gid']           = 0;
        $A['name']          = '';
        $A['is_enabled']    = 1;
        $A['php_function']  = '';
        $A['navorder']      = 0;
        $A['pattern']       = '';
        $A['class_name']    = '';
        $A['template_name'] = 'default';
        $A['var_name']      = 0;
        $A['render']        = 0;
        $A['render_for']    = 0;
        $A['opt_match']     = 0;

        $A['owner_id']      = (int) $_USER['uid'];
        if (isset($_GROUPS['Navman Admin'])) {
            $A['group_id'] = $_GROUPS['Navman Admin'];
        } else {
            $A['group_id'] = SEC_getFeatureGroup('navman.admin');
        }
        SEC_setDefaultPermissions($A, $_NAVM_CONF['default_permissions']);
        $access = 3;
    }

    $T = new Template($_NAVM_CONF['path_layout']);
    $T->set_file('editor', 'navimenueditor.thtml');
    $T->set_var('xhtml', XHTML);
    $T->set_var('site_url',       $_CONF['site_url']);
    $T->set_var('site_admin_url', $_CONF['site_admin_url']);
    $T->set_var('icon_url', plugin_geticon_navman());
    $T->set_var('layout_url',     $_CONF['layout_url']);

    $retval .= COM_startBlock($LANG_NAVM_EDITOR['navimenueditor'], '',
                              COM_getBlockTemplate ('_admin_block', 'header'));

    $v = (($mode == 'createmenu') || ($mode == 'clonemenu')) ? UC_DISABLED : '';
    $T->set_var('delete_disabled', $v);

    foreach ($LANG_NAVM_EDITOR as $key => $val) {
        $T->set_var('lang_'.$key, $val);
    }

    $v = (($mode == 'createmenu') or ($mode == 'clonemenu')) ? 'save' : 'update';
    $T->set_var('lang_save', $LANG_NAVM_EDITOR[$v]);

    $T->set_var('val_navimenuname',   stripslashes($A['name']));
    $T->set_var('val_php_function',   stripslashes($A['php_function']));
    $T->set_var('val_is_enabled',    ($A['is_enabled'] == 1) ? UC_CHECKED : '');
    $T->set_var('val_nid',            $A['nid']);
    $T->set_var('val_old_nid',        $A['nid']);
    $T->set_var('val_gid',            $A['gid']);
    $T->set_var('val_pattern',        stripslashes($A['pattern']));
    $T->set_var('val_navorder',       $A['navorder']);
    $T->set_var('val_class_name',     stripslashes($A['class_name']));
    $T->set_var('val_template_name',  stripslashes($A['template_name']));
    $T->set_var('template_name_list',       NAVM_GetTemlateVarNameList($A['var_name']));
    $T->set_var('val_render_0', ($A['render'] == 0)  ? UC_SELECTED : '');
    $T->set_var('val_render_1', ($A['render'] == 1)  ? UC_SELECTED : '');
    $T->set_var('val_render_for_0', ($A['render_for'] == 0)  ? UC_SELECTED : '');
    $T->set_var('val_render_for_1', ($A['render_for'] == 1)  ? UC_SELECTED : '');
    $T->set_var('val_render_for_2', ($A['render_for'] == 2)  ? UC_SELECTED : '');
    $T->set_var('val_opt_match_0',   ($A['opt_match'] == 0)  ? UC_SELECTED : '');
    $T->set_var('val_opt_match_1',   ($A['opt_match'] == 1)  ? UC_SELECTED : '');
    $T->set_var('val_opt_match_2',   ($A['opt_match'] == 2)  ? UC_SELECTED : '');
    $T->set_var('mitem_group_list',  NAVM_GetMenuitemGroupList($A['gid']));

    // user access info
    $T->set_var('lang_accessrights', $LANG_ACCESS['accessrights']);
    $T->set_var('lang_owner', $LANG_ACCESS['owner']);
    $ownername = COM_getDisplayName($A['owner_id']);
    $T->set_var('owner_username', DB_getItem($_TABLES['users'], 'username', "uid = {$A['owner_id']}"));
    $T->set_var('owner_name', $ownername);
    $T->set_var('owner', $ownername);
    $T->set_var('owner_id', $A['owner_id']);
    $T->set_var('lang_group', $LANG_ACCESS['group']);
    $T->set_var('group_dropdown', SEC_getGroupDropdown($A['group_id'], $access));
    $T->set_var('lang_permissions', $LANG_ACCESS['permissions']);
    $T->set_var('lang_permissionskey', $LANG_ACCESS['permissionskey']);
    $T->set_var('permissions_editor',
        SEC_getPermissionsHTML($A['perm_owner'], $A['perm_group'], $A['perm_members'], $A['perm_anon']));
    $T->set_var('lang_lockmsg', $LANG_ACCESS['permmsg']);
    if (!defined(CSRF_TOKEN)) define('CSRF_TOKEN', 'token');
    $T->set_var('gltoken_name', CSRF_TOKEN);
    $T->set_var('gltoken', SEC_createToken());

    $T->parse('output', 'editor');
    $retval .= $T->finish($T->get_var('output'));

    $retval .= COM_endBlock(COM_getBlockTemplate('_admin_block', 'footer'));

    return $retval;
}

/**
* Saves a navimenu
*
* @return   string                  HTML error message
*
*/
function NAVM_saveNaviMenu($mode)
{
    global $_TABLES, $LANG_NAVM_EDITOR, $NM;

    $retval = '';

    if ((!empty($NM['nid'])) && (DB_count($_TABLES['navmmenus'], 'nid', $NM['nid']) > 0)) {
        $result = DB_query("SELECT owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon "
                         . "FROM {$_TABLES['navmmenus']} WHERE nid = '" . addslashes($NM['nid']) . "'");
        $A = DB_fetchArray($result);
        $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                                $A['perm_owner'],   $A['perm_group'], 
                                $A['perm_members'], $A['perm_anon']);
    } else {
        $access = SEC_hasAccess($NM['owner_id'],     $NM['group_id'],
                                $NM['perm_owner'],   $NM['perm_group'],
                                $NM['perm_members'], $NM['perm_anon']);
    }

    if ($access < 3) {
        $retval .= COM_startBlock($LANG_NAVM_EDITOR['access_denied'], '', COM_getBlockTemplate('_msg_block', 'header'))
                 . $LANG_NAVM_EDITOR['message_access2']
                 . COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        COM_accessLog("User {$_USER['username']} tried to illegally submit or edit navimenu {$NM['nid']}.");
        return $retval;
    }

    $NM['nid']           = addslashes(COM_stripslashes(strip_tags($NM['nid'])));
    $NM['name']          = addslashes(COM_stripslashes(strip_tags($NM['name'])));
    $NM['php_function']  = addslashes(COM_stripslashes(strip_tags($NM['php_function'])));
    $NM['class_name']    = addslashes(COM_stripslashes(strip_tags($NM['class_name'])));
    $NM['template_name'] = addslashes(COM_stripslashes(strip_tags($NM['template_name'])));
    $NM['php_function']  = addslashes(COM_stripslashes(strip_tags($NM['php_function'])));
    $NM['pattern']       = addslashes($NM['pattern']);

    if ($mode == 'updatemenu') {
        if ($NM['old_nid'] != $NM['nid']) {
            DB_delete($_TABLES['navmmenus'], 'nid', $NM['old_nid']);
            $sql = "INSERT INTO {$_TABLES['navmmenus']} "
             .  '(nid, name, navorder, is_enabled, php_function, pattern, '
             .  'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
             .  'class_name, template_name, var_name, gid, render, render_for, opt_match) '
             .  "VALUES ('" . $NM['nid']           . "','" . $NM['name']         . "',"  . $NM['navorder']   . ","
                            . $NM['is_enabled']    . ",'"  . $NM['php_function'] . "','"
                            . $NM['pattern']       . "',"  . $NM['owner_id']     . ","
                            . $NM['group_id']      . ","   . $NM['perm_owner']   . ","   . $NM['perm_group'] . ","
                            . $NM['perm_members']  . ","   . $NM['perm_anon']    . ",'"
                            . $NM['class_name']    . "','" . $NM['template_name']. "','" . $NM['var_name']   . "',"
                            . $NM['gid']           . ","   . $NM['render']       . ","   . $NM['render_for'] . ","
                            . $NM['opt_match']     . ")";
            DB_query($sql);

        } else {
            DB_save($_TABLES['navmmenus'],
                'nid, name, navorder, is_enabled, php_function, pattern, '
              . 'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
              . 'class_name, template_name, var_name, gid, render, render_for, opt_match',
                "'" . $NM['nid']           . "','" . $NM['name']         . "',"  . $NM['navorder']      . ","
                    . $NM['is_enabled']    . ",'"  . $NM['php_function'] . "','"
                    . $NM['pattern']       . "',"  . $NM['owner_id']     . ","
                    . $NM['group_id']      . ","   . $NM['perm_owner']   . ","   . $NM['perm_group']    . ","
                    . $NM['perm_members']  . ","   . $NM['perm_anon']    . ",'"
                    . $NM['class_name']    . "','" . $NM['template_name']. "','" . $NM['var_name']      . "',"
                    . $NM['gid']           . ","   . $NM['render']       . ","   . $NM['render_for']    . ","
                    . $NM['opt_match']     . "" );
        }

    } else {
        $sql = "INSERT INTO {$_TABLES['navmmenus']} "
         .  '(nid, name, navorder, is_enabled, php_function, pattern, '
         .  'owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon, '
         .  'class_name, template_name, var_name, gid, render, render_for, opt_match) '
         .  "VALUES ('" . $NM['nid']           . "','" . $NM['name']         . "',"  . $NM['navorder']   . ","
                        . $NM['is_enabled']    . ",'"  . $NM['php_function'] . "','"
                        . $NM['pattern']       . "',"  . $NM['owner_id']     . ","
                        . $NM['group_id']      . ","   . $NM['perm_owner']   . ","   . $NM['perm_group'] . ","
                        . $NM['perm_members']  . ","   . $NM['perm_anon']    . ",'"
                        . $NM['class_name']    . "','" . $NM['template_name']. "','" . $NM['var_name']   . "',"
                        . $NM['gid']           . ","   . $NM['render']       . ","   . $NM['render_for'] . ","
                        . $NM['opt_match']     . ")";

        DB_query($sql);
    }

    return $retval;
}

/**
* Display the list of the navigation menus
*
* @return   string                  HTML error message
*
*/
function NAVM_listNaviMenus()
{
    global $_CONF, $_TABLES, $LANG_ADMIN, $LANG_NAVM_EDITOR, $LANG_NAVM, $_IMAGE_TYPE;

    require_once $_CONF['path_system'] . 'lib-admin.php';

    $retval = '';
    
    NAVM_reorderNaviMenus();
    
    $header_arr   = array(
                    array('text' => $LANG_ADMIN['edit'],       'field' => 'edit',      'sort' => false),
                    array('text' => $LANG_NAVM['order'],       'field' => 'navorder',  'sort' => true),
                    array('text' => $LANG_ADMIN['list'],       'field' => 'listitems', 'sort' => false),
                    array('text' => $LANG_ADMIN['title'],      'field' => 'name',      'sort' => true),
                    array('text' => $LANG_NAVM['id'],          'field' => 'nid',       'sort' => true),
                    array('text' => $LANG_NAVM['classname'],   'field' => 'class_name','sort' => true),
    );

    $defsort_arr = array('field' => 'navorder', 'direction' => 'asc');

    $menu_arr = array(
                    array('url'  => $_CONF['site_admin_url'] . '/plugins/navman/index.php?mode=listmenuitems',
                          'text' => $LANG_NAVM['list_menuitems']),
                    array('url'  => $_CONF['site_admin_url'] . '/plugins/navman/index.php?mode=createmenu',
                          'text' => $LANG_NAVM['create_navimenu']),
                    array('url'  => $_CONF['site_admin_url'],
                          'text' => $LANG_ADMIN['admin_home'])
    );


    $retval .= COM_startBlock($LANG_NAVM['manager'], '', COM_getBlockTemplate('_admin_block', 'header'));

    $retval .= ADMIN_createMenu($menu_arr, $LANG_NAVM['instructions'], plugin_geticon_navman());

    $text_arr = array('has_extras' => true,
                      'form_url'   => $_CONF['site_admin_url'] . "/plugins/navman/index.php");

    $query_arr = array('table'          => 'navimenus',
                       'sql'            => "SELECT * FROM {$_TABLES['navmmenus']}",
                       'query_fields'   => array('nid'),
                       'default_filter' => COM_getPermSql('AND'));

    $retval .= ADMIN_list("navimenus", "NAVM_getListField_NaviMenus", $header_arr, $text_arr,
                           $query_arr, $defsort_arr);

    $retval .= COM_endBlock(COM_getBlockTemplate('_admin_block', 'footer'));

    return $retval;
}

/**
* Get ListFields of NaviMenus
*/
function NAVM_getListField_NaviMenus($fieldname, $fieldvalue, $A, $icon_arr)
{
    global $_CONF, $LANG_ADMIN, $_PLUGINS, $_NAVM_CONF, $LANG_NAVM_EDITOR, $MESSAGE, $NAVM_CSRF_TOKEN;
    
    $retval = false;

    $access = SEC_hasAccess($A['owner_id'],     $A['group_id'],
                            $A['perm_owner'],   $A['perm_group'],
                            $A['perm_members'], $A['perm_anon']);

    if ($access > 0) {

        $token  = "";
        if (version_compare(VERSION, '1.5') >= 0) {
            $token  = "&amp;" . CSRF_TOKEN . "=" . $NAVM_CSRF_TOKEN;
        }

        switch($fieldname) {
            case 'edit':
                if ($access == 3) {
                    $retval = "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=editmenu&amp;nid={$A['nid']}\">{$icon_arr['edit']}</a>" . LB
                            . "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=clonemenu&amp;nid={$A['nid']}\">{$icon_arr['copy']}</a>" . LB;

                    $icon   = "<img src=\"{$_CONF['site_url']}/navman/images/delete.png\" "
                            . "alt=\"{$LANG_ADMIN['delete']}\" title=\"{$LANG_ADMIN['delete']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                            . "?mode=deletemenu&amp;nid={$A['nid']}"
                            . $token . "\" onclick=\"return confirm('{$MESSAGE[76]}');\">$icon</a>";
                }

                break;

            case 'navorder':
                if ($access == 3) {
                    $iconup  = "<img src=\"{$_CONF['site_url']}/navman/images/arrow-up.png\" "
                             . "alt=\"{$LANG_NAVM_EDITOR['move_up']}\" title=\"{$LANG_NAVM_EDITOR['move_up']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php?"
                             . "nid=" . $A['nid'] . "&amp;where=up" . $token . "\">$iconup</a>";

                    $icondn  = "<img src=\"{$_CONF['site_url']}/navman/images/arrow-dn.png\" "
                             . "alt=\"{$LANG_NAVM_EDITOR['move_down']}\" title=\"{$LANG_NAVM_EDITOR['move_down']}\">";

                    $retval .= "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php?"
                             . "nid=" . $A['nid'] . "&amp;where=dn" . $token . "\">$icondn</a>";

                    $retval .= "&nbsp;";
                    $retval .= $A['navorder'];
                }
                break;

            case 'listitems':
                $retval = "<a href=\"{$_CONF['site_admin_url']}/plugins/navman/index.php"
                        . "?mode=listmenuitems&amp;gid=ID{$A['gid']}\">{$icon_arr['list']}</a>" . LB;
                break;

            case 'name':
                $retval = $fieldvalue;
                if ($access == 3) {
                    $switch = ($A['is_enabled'] == 1) ? UC_CHECKED : '';
                    $val = ($A['is_enabled'] == 1) ? 1 : 0;
                    $name = 'nid_' . $A['nid'];
                    $update = 'update_' . $A['nid'];
                    $retval = "<input type=\"checkbox\" name=\"$name\" onclick=\"this.form.elements['$update'].value=1; submit()\" value=\"1\" $switch>" . $retval;
                    $retval .= "<input type=\"hidden\" name=\"$update\" value=\"0\"" . XHTML . ">";
                    $retval .= "<input type=\"hidden\" name=\"" . CSRF_TOKEN . "\" value=\"$NAVM_CSRF_TOKEN\"" . XHTML . ">";
                }
                break;

            default:
                $retval = $fieldvalue;
                break;
        }
    }
    return $retval;
}

function NAVM_GetMenuitemGroupList($current='all', $all=false, $usejs=false)
{
    global $_NAVM_CONF, $LANG_NAVM_EDITOR;

    $js = ($usejs) ? ' onchange="submit()"' : '';
    $retval = '<select name="gid"' . $js . '>';


    if ($all) {
        $selected = ($current == 'all') ? ' ' . UC_SELECTED : '';
        $retval .= '<option value="all"' . $selected . '>' . $LANG_NAVM_EDITOR['all'] . '</option>';
    }

    foreach ($_NAVM_CONF['menuitem_group'] as $key => $val) {
        $selected = (((string) $current) == ((string) $key)) ? ' ' . UC_SELECTED : '';
        $retval .= '<option value="ID' . $key . '"' . $selected . '>' . $val . '</option>';
    }
    $retval .= '</select>';
    
    return $retval;
}

function NAVM_GetTemlateVarNameList($current)
{
    global $_NAVM_CONF, $LANG_NAVM_EDITOR;

    $retval = '<select name="var_name">';

    foreach ($_NAVM_CONF['templatevar'] as $key => $val) {
        $selected = (((string) $current) == ((string) $key)) ? ' ' . UC_SELECTED : '';
        $retval .= '<option value="' . $key . '"' . $selected . '>' . $val . '</option>';
    }
    $retval .= '</select>';
    
    return $retval;
}


// MAIN

$mode = (!empty($_REQUEST['mode'])) ? $_REQUEST['mode'] : '';
$mode = (($mode == $LANG_ADMIN['save'])         && !empty($LANG_ADMIN['save']))         ? 'save'   : $mode;
$mode = (($mode == $LANG_ADMIN['delete'])       && !empty($LANG_ADMIN['delete']))       ? 'delete' : $mode;
$mode = (($mode == $LANG_ADMIN['cancel'])       && !empty($LANG_ADMIN['cancel']))       ? 'cancel' : $mode;
$mode = (($mode == $LANG_NAVM_EDITOR['update']) && !empty($LANG_NAVM_EDITOR['update'])) ? 'update' : $mode;

$nid = (!empty($_REQUEST['nid'])) ? COM_applyFilter($_REQUEST['nid']) : '';
$mid = (!empty($_REQUEST['mid'])) ? COM_applyFilter($_REQUEST['mid']) : '';


if (!isset($_REQUEST['gid'])) {
    $gid = 'all';
} else {
    $gid = (!empty($_REQUEST['gid'])) ? COM_applyFilter($_REQUEST['gid']) : 'all';
    $gid = str_replace('ID', '', $gid);
}

$target = (!empty($_POST['target'])) ? COM_applyFilter($_POST['target']) : '';
$mode = (($target == 'navimenu') && ($mode == 'save'))   ? 'savemenu'   : $mode;
$mode = (($target == 'navimenu') && ($mode == 'delete')) ? 'deletemenu' : $mode;
$mode = (($target == 'navimenu') && ($mode == 'update')) ? 'updatemenu' : $mode;

NAVM_changeMenuitemStatus();
NAVM_changeNaviMenuStatus();

$where = COM_applyFilter($_GET['where']);
if (!empty($where) && SEC_checkToken()) {
    if ($mode == 'listmenuitems') {
        NAVM_moveMenuitem($gid);
    } else {
        NAVM_moveNaviMenu($nid);
    }
}

if ($mode == 'delete' && SEC_checkToken()) {

    NAVM_setMI();
    if (!empty($MI['old_mid'])) $mid = $MI['old_mid'];
    if (!isset ($mid) || empty ($mid)) {
        COM_errorLog('Navman admin: Attempted to delete menuitem, mid empty or null, value =' . $mid);
        $display .= COM_refresh($_CONF['site_admin_url'] . '/plugins/navman/index.php');
    } else {
        $msg = NAVM_deleteMenuitem($mid);
        $NAVM_CSRF_TOKEN = SEC_createToken();
        $display .= COM_siteHeader('menu', $LANG_NAVM['manager']);
        $display .= $msg;
        $display .= NAVM_listMenuitems($gid);
        $display .= COM_siteFooter();
    }

} else if ($mode == 'deletemenu' && SEC_checkToken()) {

    NAVM_setNM();
    if (!empty($NM['old_nid'])) $nid = $NM['old_nid'];
    if (!isset ($nid) || empty ($nid)) {
        COM_errorLog('Navman admin: Attempted to delete navimenu, nid empty or null, value =' . $nid);
        $display .= COM_refresh($_CONF['site_admin_url'] . '/plugins/navman/index.php');
    } else {
        $msg = NAVM_deleteNaviMenu($nid);
        $NAVM_CSRF_TOKEN = SEC_createToken();
        $display .= COM_siteHeader('menu', $LANG_NAVM['manager']);
        $display .= $msg;
        $display .= NAVM_listNaviMenus();
        $display .= COM_siteFooter();
    }

} else if ((($mode == 'save') || ($mode == 'update')) && SEC_checkToken()) {

    NAVM_setMI();
    $msg = NAVM_validateMI($mode);
    if (empty($msg)) {
        $msg = NAVM_saveMenuitems($mode);
        $NAVM_CSRF_TOKEN = SEC_createToken();
        $display .= COM_siteHeader('menu', $LANG_NAVM['manager']);
        $display .= $msg;
        $display .= NAVM_listMenuitems($gid);
        $display .= COM_siteFooter();
    } else {
        $display .= COM_siteHeader('menu', $LANG_NAVM_EDITOR['navmaneditor']);
        $display .= COM_startBlock($LANG_NAVM_EDITOR['error_field'], '', COM_getBlockTemplate('_msg_block', 'header'));
        $display .= $msg;
        $display .= COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        $display .= NAVM_editMenuitem($mid, 'edit', $MI);
        $display .= COM_siteFooter();
    }

} else if ((($mode == 'savemenu') || ($mode == 'updatemenu')) && SEC_checkToken()) {

    NAVM_setNM();
    $msg = NAVM_validateNM($mode);
    if (empty($msg)) {
        $msg = NAVM_saveNaviMenu($mode);
        $NAVM_CSRF_TOKEN = SEC_createToken();
        $display .= COM_siteHeader('menu', $LANG_NAVM['manager']);
        $display .= $msg;
        $display .= NAVM_listNaviMenus();
        $display .= COM_siteFooter();
    } else {
        $display .= COM_siteHeader('menu', $LANG_NAVM_EDITOR['navmaneditor']);
        $display .= COM_startBlock($LANG_NAVM_EDITOR['error_field'], '', COM_getBlockTemplate('_msg_block', 'header'));
        $display .= $msg;
        $display .= COM_endBlock(COM_getBlockTemplate('_msg_block', 'footer'));
        $display .= NAVM_editNaviMenu($nid, 'editmenu', $NM);
        $display .= COM_siteFooter();
    }

} else if (($mode == 'create') || ($mode == 'edit') || ($mode == 'clone')) {

    $display .= COM_siteHeader('menu', $LANG_NAVM_EDITOR['navmaneditor']);
    $display .= NAVM_editMenuitem($mid, $mode);
    $display .= COM_siteFooter();

} else if (($mode == 'createmenu') || ($mode == 'editmenu') || ($mode == 'clonemenu')) {

    $display .= COM_siteHeader('menu', $LANG_NAVM_EDITOR['navmaneditor']);
    $display .= NAVM_editNaviMenu($nid, $mode);
    $display .= COM_siteFooter();

} else {

    if (($mode != 'cancel') && ($mode != 'move')) {
        NAVM_addPluginsMenuitems();
    }
    $NAVM_CSRF_TOKEN = SEC_createToken();
    $display .= COM_siteHeader('menu', $LANG_NAVM['manager']);

    if ($mode == 'listmenuitems') {
        $display .= NAVM_listMenuitems($gid);
    } else {
        $display .= NAVM_listNaviMenus();
    }

    $display .= COM_siteFooter();
}

COM_output($display);
?>
