<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Navigation Manager - Navman Plugin for Geeklog                            |
// +---------------------------------------------------------------------------+
// | plugins/navman/autoinstall.php                                            |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 dengen - taharaxp AT gmail DOT com                     |
// |                                                                           |
// | Constructed with the Universal Plugin                                     |
// | Copyright (C) 2002 by the following authors:                              |
// | Tom Willett               -    twillett AT users DOT sourceforge DOT net  |
// | Blaine Lang               -    langmail AT sympatico DOT ca               |
// | The Universal Plugin is based on prior work by:                           |
// | Tony Bibbs                -    tony AT tonybibbs DOT com                  |
// | Modified by:                                                              |
// | mystral-kk                -    geeklog AT mystral-kk DOT net              |
// | dengen                    -    taharaxp AT gmail DOT com                  |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

function plugin_autoinstall_navman($pi_name)
{
    $pi_name         = 'navman';
    $pi_display_name = 'Navman';
    $pi_admin        = $pi_display_name . ' Admin';

    $info = array(
        'pi_name'         => $pi_name,
        'pi_display_name' => $pi_display_name,
        'pi_version'      => '0.2.0',
        'pi_gl_version'   => '1.6.0',
        'pi_homepage'     => 'http://www.trybase.com/~dengen/log/'
    );

    $groups = array(
        $pi_admin => 'Has full access to ' . $pi_display_name . ' features'
    );

    $features = array(
        $pi_name . '.admin'  => 'Navman Admin'
    );

    $mappings = array(
        $pi_name . '.admin'  => array($pi_admin)
    );

    $tables = array(
        'navmmenus',
        'navmmenuitems',
    );

    $inst_parms = array(
        'info'      => $info,
        'groups'    => $groups,
        'features'  => $features,
        'mappings'  => $mappings,
        'tables'    => $tables
    );

    return $inst_parms;
}

function plugin_load_configuration_navman($pi_name)
{
    global $_CONF;

    $base_path = $_CONF['path'] . 'plugins/' . $pi_name . '/';

    require_once $_CONF['path_system'] . 'classes/config.class.php';
    require_once $base_path . 'install_defaults.php';

    return plugin_initconfig_navman();
}

function plugin_postinstall_navman($pi_name)
{
    return true;
}

function plugin_compatible_with_this_version_navman($pi_name)
{
    if (function_exists('COM_printUpcomingEvents')) {
        // if this function exists, then someone's trying to install the
        // plugin on Geeklog 1.4.0 or older - sorry, but that won't work
        return false;
    }   

    if (!function_exists('MBYTE_strpos')) {
        // the plugin requires the multi-byte functions
        return false; 
    }   

    if (!function_exists('COM_createLink')) {
        return false;
    }

    if (!function_exists('SEC_createToken')) {
        return false;
    }

    if (!function_exists('COM_showMessageText')) {
        return false;
    }

    return true;
}

/**
* Automatic uninstall function for plugins
*
* @return   array
*
* This code is automatically uninstalling the plugin.
* It passes an array to the core code function that removes
* tables, groups, features and php blocks from the tables.
* Additionally, this code can perform special actions that cannot be
* foreseen by the core code (interactions with other plugins for example)
*
*/
function NAVM_autouninstall()
{
    $out = array(
        // give the name of the tables, without $_TABLES[]
        'tables' => array('navmmenus', 'navmmenuitems'),

        // give the full name of the group, as in the db
        'groups' => array('Navman Admin'),

        // give the full name of the feature, as in the db
        'features' => array('navman.admin'),
    );
    return $out;
}

/**
* Called by the plugin Editor to run the SQL Update for a plugin update
*/
function NAVM_upgrade()
{
    global $_TABLES;

    $pi_name = 'navman';
    $installed_version = DB_getItem($_TABLES['plugins'], 'pi_version', "pi_name = '$pi_name'");
    $code_version = plugin_chkVersion_navman();
    if ($installed_version == $code_version) return true;
    if (!plugin_compatible_with_this_version_navman($pi_name)) return 3002;

    $inst_parms = plugin_autoinstall_navman($pi_name);
    $pi_gl_version = $inst_parms['info']['pi_gl_version'];

    $current_version = $installed_version;
    if (version_compare($current_version, '0.2.0', '<')) {
        $success = NAVM_upgrade_020();
        if (!$success) return 3011;
        $current_version = '0.2.0';
    }

    // Update the version numbers
    DB_query("UPDATE {$_TABLES['plugins']} "
           . "SET pi_version = '$code_version', pi_gl_version = '$pi_gl_version' "
           . "WHERE pi_name = '$pi_name'");

    COM_errorLog(ucfirst($pi_name)
        . " plugin was successfully updated to version $code_version.");

    return true;
}

function NAVM_upgrade_020()
{
    global $_TABLES, $_USER;

    $result = DB_query("SELECT max(menuorder) FROM {$_TABLES['navmmenuitems']}");
    list($menuorder) = DB_fetchArray($result);
    $order = array();
    for ($i=0; $i<6; $i++) $order[$i] = $menuorder + ($i + 1) * 10;
    $grp_id = DB_getItem($_TABLES['groups'], 'grp_id',"grp_name = 'Navman Admin'");

    $postfix = '';
    do {
        $c = 0;
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_downloads' . $postfix);
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_downloads_category' . $postfix);
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_downloads_file' . $postfix);
        if ($c == 0) break;
        $postfix = $postfix + 1;
    } while ($c > 0);
    $bc_downloads          = 'bc_downloads' . $postfix;
    $bc_downloads_category = 'bc_downloads_category' . $postfix;
    $bc_downloads_file     = 'bc_downloads_file' . $postfix;

    $postfix = '';
    do {
        $c = 0;
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_mediagallery' . $postfix);
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_mediagallery_album' . $postfix);
        $c += DB_count($_TABLES['navmmenuitems'], "mid", 'bc_mediagallery_media' . $postfix);
        if ($c == 0) break;
        $postfix = $postfix + 1;
    } while ($c > 0);
    $bc_mediagallery       = 'bc_mediagallery' . $postfix;
    $bc_mediagallery_album = 'bc_mediagallery_album' . $postfix;
    $bc_mediagallery_media = 'bc_mediagallery_media' . $postfix;

    $_SQL = array();
    $_SQL[] = "INSERT INTO {$_TABLES['navmmenuitems']} (mid, pmid, gid, is_enabled, type, label, label_var, php_function, url, icon_url, class_name, tid, menuorder, pattern, opt_match, owner_id, group_id, contentid) VALUES 
('$bc_downloads', '', 2, 1, 'custom', '[BC] Downloads', 'LANG_DLM[''plugin_name'']', '', '[site_url]/downloads/index.php', '', '', 'all', {$order[0]}, '@(?<!plugins)/downloads/index\\\\\\\\.php$@', 2, {$_USER['uid']}, '$grp_id', ''),
('$bc_downloads_category', '$bc_downloads', 2, 1, 'custom', '[BC] Downloads category', '', 'bc_downloads_category', '', '', '', 'all', {$order[1]}, '/downloads/index.php?cid=', 0, {$_USER['uid']}, '$grp_id', ''),
('$bc_downloads_file', '$bc_downloads_category', 2, 1, 'custom', '[BC] Downloads file', '', 'bc_downloads_file', '', '', '', 'all', {$order[2]}, '@/downloads/index\\\\\\\\.php\\\\\\\\?id=|/downloads/index\\\\\\\\.php/@', 2, {$_USER['uid']}, '$grp_id', ''),
('$bc_mediagallery', '', 2, 1, 'custom', '[BC] Mediagallery', 'LANG_MG00[''plugin'']', '', '[site_url]/mediagallery/index.php', '', '', 'all', {$order[3]}, '@(?<!plugins)/mediagallery/index\\\\\\\\.php@', 2, {$_USER['uid']}, '$grp_id', ''),
('$bc_mediagallery_album', '$bc_mediagallery', 2, 1, 'custom', '[BC] Mediagallery album', '', 'bc_mediagallery_album', '', '', '', 'all', {$order[4]}, '/mediagallery/album.php', 0, {$_USER['uid']}, '$grp_id', ''),
('$bc_mediagallery_media', '$bc_mediagallery_album', 2, 1, 'custom', '[BC] Mediagallery media', '', 'bc_mediagallery_media', '', '', '', 'all', {$order[5]}, '/mediagallery/media.php', 0, {$_USER['uid']}, '$grp_id', '');";

    for ($i=0; $i < count($_SQL); $i++) {
        DB_query(current($_SQL));
        if (DB_error()) {
            COM_errorLog("SQL Error during Navman plugin 0.2.0 update: SQL: " . current($_SQL));
            return false;
        }
        next($_SQL);
    }

    return true;
}

?>
